/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to you under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.calcite.sql.type;

import org.apache.calcite.rel.type.RelDataType;
import org.apache.calcite.rel.type.RelDataTypeSystem;

import com.google.common.collect.Lists;

import org.junit.Test;

import static org.junit.Assert.assertEquals;

/**
 * Test for {@link SqlTypeFactoryImpl}
 */
public class SqlTypeFactoryTest {

  @Test
  public void testLeastRestrictiveWithAny() {
    SqlTypeFactoryImpl typeFactory = new SqlTypeFactoryImpl(RelDataTypeSystem.DEFAULT);
    final RelDataType sqlBigInt = typeFactory.createSqlType(SqlTypeName.BIGINT);
    final RelDataType sqlAny = typeFactory.createSqlType(SqlTypeName.ANY);

    RelDataType leastRestrictive =
            typeFactory.leastRestrictive(Lists.newArrayList(sqlBigInt, sqlAny));
    assertEquals(leastRestrictive.getSqlTypeName(), SqlTypeName.ANY);
  }

  @Test
  public void testLeastRestrictiveWithNumbers() {
    SqlTypeFactoryImpl typeFactory = new SqlTypeFactoryImpl(RelDataTypeSystem.DEFAULT);
    final RelDataType sqlBigInt = typeFactory.createSqlType(SqlTypeName.BIGINT);
    final RelDataType sqlInt = typeFactory.createSqlType(SqlTypeName.INTEGER);

    RelDataType leastRestrictive =
            typeFactory.leastRestrictive(Lists.newArrayList(sqlBigInt, sqlInt));
    assertEquals(leastRestrictive.getSqlTypeName(), SqlTypeName.BIGINT);
  }
}
// End SqlTypeFactoryTest.java
